# MistAws

Uses the [ruby aws-sdk v2](https://github.com/aws/aws-sdk-core-ruby) [Resource Interface](https://github.com/aws/aws-sdk-core-ruby#resource-interfaces) to create some use specific higher level functionality.

> NOTE: Currently the only object implemented is Iam for creating / deleting instance roles and an Ec2 for working with Security Groups

[![mist_aws API Documentation](https://www.omniref.com/ruby/gems/mist_aws.png)](https://www.omniref.com/ruby/gems/mist_aws)

## Installation

### Install the Gem

Add this line to your application's Gemfile:

```ruby
gem 'mist_aws'
```

And then execute:

    $ bundle

Or install it yourself as:

    $ gem install mist_aws

### Generate Yard Docs

    $ rake yard
    $ open doc/index.html

## Usage

* Get the handle to the MistAws::Iam object
    * Credentials will come from the `my_profile_name` profile in ~/.aws/credentials as per [Providing AWS Credentials in the AWS SDK for Ruby] (http://docs.aws.amazon.com/AWSSdkDocsRuby/latest/DeveloperGuide/prog-basics-creds.html#creds-explicit)  
```
mist_aws = ::MistAws::Iam.new(profile_name: "my_profile_name", region: "us-east-1")
```
* Create an IAM Role
    * role_policy_document is a JSON text file that is an [AWS IAM policy](http://docs.aws.amazon.com/IAM/latest/UserGuide/PoliciesOverview.html)
    * This method will create:
        * [Role](http://docs.aws.amazon.com/sdkforruby/api/Aws/IAM/Role.html)
        * [Role Policy](http://docs.aws.amazon.com/sdkforruby/api/Aws/IAM/RolePolicy.html)
        * [Instance profile](http://docs.aws.amazon.com/sdkforruby/api/Aws/IAM/InstanceProfile.html)
        * Connect them all together
    * Returns a [Role](http://docs.aws.amazon.com/sdkforruby/api/Aws/IAM/Role.html) Resource instance
```
role = mist_aws.create_iam_role(role_name, role_policy_name, role_policy_document, instance_profile_name)
```

See yard docs for more info.

## Contributing

I would be very interested in feedback on things that could be done
better. Also would be interested in how to better do the rspec
tests. I felt like there was WAY too much mocking. I did find it
useful for finding bugs and broken assumptions though.


1. Fork it ( https://github.com/[my-github-username]/mist_aws/fork )
2. Create your feature branch (`git checkout -b my-new-feature`)
3. Commit your changes (`git commit -am 'Add some feature'`)
4. Push to the branch (`git push origin my-new-feature`)
5. Create a new Pull Request
