# These tests actually hit AWS and are not mocked

require 'spec_helper'
require 'json'

include MistAws

describe "Live tests on AWS (not mocked)" do
  describe Ec2 do
    PRE = "trsh_#{ENV['USER']}"
    let(:profile_name) { 'mistsys' }
    let(:region) { 'us-east-1' }
    let(:cdr_block) { "192.168.99.0/24" }
    let(:test_group) { "#{PRE}_test_group" }
    let(:credentials) { ::Aws::SharedCredentials.new(profile_name: profile_name) }
    let(:ec2_client) { ::Aws::EC2::Client.new(credentials: credentials, region: region) }
    let(:my_mist_aws) { ::MistAws::Ec2.new(profile_name: profile_name, region: region) }
    let!(:vpc) { my_mist_aws.create_vpc(cdr_block) }
    let!(:vpc_id) { vpc.vpc_id }

    it 'has a version number' do
      expect(MistAws::VERSION).not_to be nil
    end

    describe '#security_group_exists?' do
      before(:each) do
        my_mist_aws.delete_security_group(test_group, vpc_id)
      end

      after(:each) do
        my_mist_aws.delete_security_group(test_group, vpc_id) 
      end

      it 'is falsy when it does not exist' do
        expect(my_mist_aws.security_group_exists?(test_group, vpc_id)).to be_falsy
      end

      it 'is truthy when it does exist' do
        my_mist_aws.create_security_group(test_group, vpc_id)
        expect(my_mist_aws.security_group_exists?(test_group, vpc_id)).to be_truthy
      end
    end

    describe '#authorize_security_group_ingress' do
      let (:my_security_group) { my_mist_aws.create_security_group(test_group, vpc_id) }
      let (:my_security_group_id) { my_security_group.group_id }

      after(:each) do
        my_mist_aws.delete_security_group(test_group, vpc_id) 
      end

      it 'should not blow up' do
        opts = {
            group_id: my_security_group_id,
            ip_protocol: "tcp", 
            from_port: 22, 
            to_port: 22, 
            cidr_ip: "0.0.0.0/0"
          }
        expect { 
          my_mist_aws.authorize_security_group_ingress(opts)
        }.not_to raise_error
      end
    end
  end
end
