# These tests actually hit AWS and are not mocked

require 'spec_helper'
require 'json'

include MistAws

describe "Live tests on AWS (not mocked)" do
  describe Iam do
    PRE = "trsh_#{ENV['USER']}"
    let(:profile_name) { 'mistsys' }
    let(:region) { 'us-east-1' }
    let(:role_name) { "#{PRE}_my_role" }
    let(:role_policy_name) { "#{PRE}_my_role_policy_name" }
    let(:instance_profile_name) { "#{PRE}_my_instance_profile" }
    let(:test_dir) { File.dirname(__FILE__) }
    let(:role_policy_document_filename) { "#{test_dir}/policy_document.txt" }
    let(:role_policy_document) { File.readlines(role_policy_document_filename).join("\n") }
    let(:assume_role_policy)  {
      {
        "Version" => "2012-10-17",
        "Statement" => [
          {"Effect" => "Allow",
            "Principal" => {
              "Service" => [
                "ec2.amazonaws.com"
              ]
            },
            "Action" => ["sts:AssumeRole"]
          }
        ]
      }.to_json
    }

    let(:test_credentials) { ::Aws::SharedCredentials.new(profile_name: profile_name) }
    let(:test_iam_client) { ::Aws::IAM::Client.new(credentials: test_credentials, region: region) }
    let(:test_iam) { ::Aws::IAM::Resource.new(client: test_iam_client) }

    let(:mist_aws) { ::MistAws::Iam.new(profile_name: profile_name, region: region) } 

    describe '#create_iam_role' do
      context 'when no role already exists' do
        before(:each) do
          allow(mist_aws.logger).to receive(:warn).with(/No role/)
          mist_aws.delete_iam_role(role_name, role_policy_name, instance_profile_name)
        end

        after(:each) do
          mist_aws.delete_iam_role(role_name, role_policy_name, instance_profile_name)
        end

        it 'creates a role' do
          test_role = mist_aws.create_iam_role(role_name, role_policy_name, role_policy_document, instance_profile_name)
          # test_role = test_iam.role role_name
          expect(test_role.name).to eq role_name
          expect(test_role.assume_role_policy_document).to include 'Statement'
        end
      end
    end

    context 'when a role already exists' do
      before(:each) do
        allow(mist_aws.logger).to receive(:warn).with(/already exists/)
        mist_aws.create_iam_role(role_name, role_policy_name, role_policy_document, instance_profile_name)
      end

      after(:each) do
        mist_aws.delete_iam_role(role_name, role_policy_name, instance_profile_name)
      end

      it 'returns falsy' do
        expect(mist_aws.role_exists? role_name).to be_truthy
        expect(mist_aws.create_iam_role(role_name, role_policy_name, role_policy_document, instance_profile_name)).to be_falsy
      end
    end
  end
end
