# These should be purly mocked and not hit AWS

require 'spec_helper'
require 'json'

include MistAws

describe "Fully Mocked Tests" do
  describe Iam do
    it 'has a version number' do
      expect(MistAws::VERSION).not_to be nil
    end


    let(:my_profile_name) { 'arg_specified_profile_name' }
    let(:env_profile_name) { 'env_profile_name' }
    let(:access_key_value) { 'access_key_value' }
    let(:default_region) { 'us-east-1' }

    let(:mist_aws) { ::MistAws::Iam.new } 
    let(:role_name) { "my_role" }
    let(:role_policy_name) { "my_role_policy_name" }
    let(:role_policy_document) { "my_role_policy_document" }
    let(:instance_profile_name) { "my_instance_profile" }

    let (:shared_credentials) { instance_double("::Aws::SharedCredentials") }
    #let(:credentials) {object_double(::Aws::SharedCredentials.new(profile_name: 'default')) }
    let(:iam_client) { object_double(::Aws::IAM::Client.new(credentials: shared_credentials, region: 'us-east-1')) }
    let(:iam) { object_double(::Aws::IAM::Resource.new(client: iam_client)) }
    let(:context) { object_double(::Seahorse::Client::RequestContext) }

    let(:role_policy) { instance_double("Aws::IAM::RolePolicy", role_name: role_name, name: role_policy_name) }
    let(:instance_profile) { instance_double("Aws::IAM::InstanceProfile",name: instance_profile_name) }

    let(:assume_role_policy)  {
      {
        "Version" => "2012-10-17",
        "Statement" => [
          {"Effect" => "Allow",
            "Principal" => {
              "Service" => [
                "ec2.amazonaws.com"
              ]
            },
            "Action" => ["sts:AssumeRole"]
          }
        ]
      }.to_json
    }

    let(:role) { object_double(::Aws::IAM::Role.new(name: role_name, client: iam_client)) }

    before(:example, :run_global_before) do
      ENV['AWS_PROFILE'] = ENV['AWS_REGION'] = ENV['AWS_ACCESS_KEY_ID'] = ENV['AWS_SECRET_ACCESS_KEY']= nil
      allow_any_instance_of(::MistAws::Iam).to receive(:get_creds).and_return(shared_credentials)
      allow(::Aws::IAM::Client).to receive(:new).with(credentials: shared_credentials, region: default_region).and_return(iam_client)
      allow(::Aws::IAM::Resource).to receive(:new).with(client: iam_client).and_return(iam)
    end
    
    after(:example) do
      ENV['RSPEC_IGNORE_INITIALIZE'] = nil
    end

    describe '#get_creds' do
      before(:example) do
        ENV['RSPEC_IGNORE_INITIALIZE'] = 'true'
        ENV['AWS_PROFILE'] = ENV['AWS_REGION'] = ENV['AWS_ACCESS_KEY_ID'] = ENV['AWS_SECRET_ACCESS_KEY']= nil
      end

      context 'when all inputs are nil' do
        it 'will fetch the default shared credential' do
          expect(::Aws::SharedCredentials).to receive(:new).with(profile_name: 'default')
          my_mist_aws = ::MistAws::Iam.new
          my_mist_aws.get_creds
        end
      end

      context 'profile is specified in arguments' do
        
        it 'will fetch the profile specified by argument' do
          expect(::Aws::SharedCredentials).to receive(:new).with(profile_name: my_profile_name)
          mist_aws.get_creds(my_profile_name)
        end


        it 'will ignore ENV["AWS_PROFILE"]' do
          ENV['AWS_PROFILE'] = 'invalid_profile_name'
          expect(::Aws::SharedCredentials).to receive(:new).with(profile_name: my_profile_name)
          mist_aws.get_creds(my_profile_name)
        end
        it 'will ignore ENV["AWS_ACCESS_KEY_ID"], ENV["AWS_SECRET_ACCESS_KEY"]' do
          ENV['AWS_ACCESS_KEY_ID'] = ENV['AWS_SECRET_ACCESS_KEY'] = access_key_value
          expect(::Aws::SharedCredentials).to receive(:new).with(profile_name: my_profile_name)
          mist_aws.get_creds(my_profile_name)
        end
      end

      context 'profile only specified in ENV["AWS_PROFILE"]' do
        before(:example) do
          ENV['AWS_PROFILE'] = env_profile_name
          ENV['AWS_ACCESS_KEY_ID'] = ENV['AWS_SECRET_ACCESS_KEY'] = access_key_value
        end

        it 'will fetch the profile specified by ENV["AWS_PROFILE\`]' do
          expect(::Aws::SharedCredentials).to receive(:new).with(profile_name: env_profile_name)
          mist_aws.get_creds
        end
        
        context "only ENV['AWS_ACCESS_KEY_ID'] and ENV['AWS_SECRET_ACCESS_KEY'] are set" do
          before(:example) do
            ENV['AWS_PROFILE'] = nil
          end
          it 'will use Aws::Credentials and not Aws::SharedCredentials' do
            expect(::Aws::SharedCredentials).not_to receive(:new)
            expect(::Aws::Credentials).to receive(:new).with(access_key_value, access_key_value)

            mist_aws.get_creds
          end
        end
      end
    end

    describe '.initialize', :run_global_before do
      context 'when all inputs are nil' do
        before(:example) do
          expect_any_instance_of(Iam).to receive(:get_creds).with(nil).and_return(shared_credentials)
        end

        it 'initializes an IAM Client' do
          expect(Aws::IAM::Client).to receive(:new).with({credentials: shared_credentials, region: default_region})
          Iam.new
        end
      end
    end

    describe '#role_exists?', :run_global_before do
      it 'returns truthy when it exists' do
        expect(iam_client).to receive(:get_role).with(role_name: role_name).and_return(role)

        expect(mist_aws.role_exists?(role_name)).to be_truthy
      end

      it 'returns falsy when it does not exist' do
        expect(iam_client).to receive(:get_role).with(role_name: role_name).and_raise(Aws::IAM::Errors::NoSuchEntity.new(context, 'foo'))

        expect(mist_aws.role_exists?(role_name)).to be_falsy
      end
    end

    describe '#instance_profile_exists?', :run_global_before do
      it 'returns truthy when it exists' do
        expect(iam).to receive(:instance_profile).with(instance_profile_name).and_return(instance_profile)
        expect(instance_profile).to receive(:data_loaded?).and_return(true)

        expect(mist_aws.instance_profile_exists?(instance_profile_name)).to be_truthy
      end

      it 'returns falsy when it does not exist' do
        expect(iam).to receive(:instance_profile).with(instance_profile_name).and_return(instance_profile)
        expect(instance_profile).to receive(:data_loaded?).and_return(false)

        expect(mist_aws.instance_profile_exists?(instance_profile_name)).to be_falsy
      end
    end

    describe '#role_policy_exists?', :run_global_before do
      it 'returns truthy when it exists' do
        expect(iam_client).to receive(:get_role_policy).with(role_name: role_name,  policy_name: role_policy_name).and_return(role_policy)

        expect(mist_aws.role_policy_exists?(role_name, role_policy_name)).to be_truthy
      end

      it 'returns falsy when it does not exist' do
        expect(mist_aws.logger).to receive(:warn).with(/No role/)
        expect(iam_client).to receive(:get_role_policy).with(role_name: role_name, policy_name: role_policy_name).and_raise(Aws::IAM::Errors::NoSuchEntityException.new(context, 'foo'))

        expect(mist_aws.role_policy_exists?(role_name, role_policy_name)).to be_falsy
      end
    end
    
    describe '#create_iam_role', :run_global_before do
      it 'creates an IAM role with a role policy and an instance_profile' do
        expect_any_instance_of(::MistAws::Iam).to receive(:role_exists?).with(role_name).and_return(false)
        expect(iam).to receive(:create_role).with(role_name: role_name, assume_role_policy_document: assume_role_policy).and_return(role)

        expect(::Aws::IAM::RolePolicy).to receive(:new).with(role_name: role_name, name: role_policy_name, client: iam_client).and_return(role_policy)
        expect(role_policy).to receive(:put).with(policy_document: role_policy_document)

        expect(iam).to receive(:instance_profile).with(instance_profile_name).and_return(instance_profile)
        expect(instance_profile).to receive(:add_role).with(role_name: role_name)
        expect(instance_profile).to receive(:data_loaded?).and_return(false)
        expect(iam).to receive(:create_instance_profile).with(instance_profile_name: instance_profile_name).and_return(instance_profile)

        
        mist_aws.create_iam_role(role_name, role_policy_name, role_policy_document, instance_profile_name)
      end

    end
    
    describe '#delete_iam_role', :run_global_before do
      it 'Does nothing if there is no role' do
        expect(mist_aws.logger).to receive(:warn).with(/No role/)
        expect(iam_client).to receive(:get_role).with(role_name: role_name).and_raise(Aws::IAM::Errors::NoSuchEntity.new(context, 'foo'))
        
        expect(mist_aws.delete_iam_role(role_name, role_policy_name, instance_profile_name)).to be_falsy
      end

      it 'Existing role, policy and instance_profile' do
        expect(iam_client).to receive(:get_role).with(role_name: role_name).and_return(role)
        expect(iam).to receive(:role).with(role_name).and_return(role)

        # Get all the instance_profiles associated with the role
        expect(role).to receive(:instance_profiles).and_return([instance_profile])

        # Removes the role from the instance_profile
        expect(instance_profile).to receive(:remove_role).with(role_name: role_name)
        # Delete the instance_profile
        expect(instance_profile).to receive(:delete)
        
        # Get all role_policies associated with the role
        expect(role).to receive(:policies).and_return([role_policy])
        
        # Deletes the role_policy
        expect(role_policy).to receive(:delete)
        
        # Deletes the role
        expect(role).to receive(:delete)
        
        expect(mist_aws.delete_iam_role(role_name, role_policy_name, instance_profile_name)).to be_truthy
      end
    end
  end
end
